@@ -1,5 +1,5 @@
 package Form::Tiny::Meta;
-$Form::Tiny::Meta::VERSION = '2.17';
+$Form::Tiny::Meta::VERSION = '2.18';
 use v5.10;
 use strict;
 use warnings;
@@ -235,7 +235,7 @@ sub add_field
 		$scalar_param = {@parameters, name => $scalar_param};
 	}
 
-	my $builder = Form::Tiny::FieldDefinitionBuilder->new(data => $scalar_param)->build;
+	my $builder = Form::Tiny::FieldDefinitionBuilder->new(build_data => $scalar_param)->build;
 	push @{$self->fields}, $builder;
 
 	$self->set_dynamic(1)
