@@ -1,5 +1,5 @@
 package Form::Tiny::Plugin::Strict;
-$Form::Tiny::Plugin::Strict::VERSION = '2.17';
+$Form::Tiny::Plugin::Strict::VERSION = '2.18';
 use v5.10;
 use strict;
 use warnings;
@@ -21,36 +21,32 @@ sub plugin
 use Moo::Role;
 
 use constant {
-	MARKER_NONE => '',
-	MARKER_SKIP => 'skip',
-	MARKER_ARRAY => 'array',
-	MARKER_LEAF => 'leaf',
+	MARKER_NONE => 0,
+	MARKER_ARRAY => 1,
+	MARKER_LEAF => 2,
 };
 
 requires qw(setup);
 
 sub _check_recursive
 {
-	my ($self, $obj, $data, $markers, $path) = @_;
-	$path //= Form::Tiny::Path->empty;
+	my ($self, $data, $markers, $path) = @_;
 
 	my $current_path = $path->join;
 	my $metadata = $markers->{$current_path} // MARKER_NONE;
 
-	return if $metadata eq MARKER_SKIP;
-
-	if ($metadata eq MARKER_LEAF) {
+	if ($metadata == MARKER_LEAF) {
 
 		# we're at leaf and no error occured - we're good.
 	}
 
-	elsif ($metadata eq MARKER_ARRAY) {
+	elsif ($metadata == MARKER_ARRAY) {
 		die $current_path unless ref $data eq 'ARRAY';
+
+		# no need to clone el for each array element
+		my $subel_path = $path->clone->append('ARRAY');
 		foreach my $value (@$data) {
-			$self->_check_recursive(
-				$obj, $value, $markers,
-				$path->clone->append('ARRAY')
-			);
+			$self->_check_recursive($value, $markers, $subel_path);
 		}
 	}
 
@@ -60,7 +56,7 @@ sub _check_recursive
 		die $current_path unless ref $data eq 'HASH';
 		for my $key (keys %$data) {
 			$self->_check_recursive(
-				$obj, $data->{$key}, $markers,
+				$data->{$key}, $markers,
 				$path->clone->append(HASH => $key)
 			);
 		}
@@ -73,24 +69,19 @@ sub _check_strict
 
 	my %markers;
 	foreach my $def (@{$obj->field_defs}) {
-		if ($def->is_subform) {
-			$markers{$def->name} = MARKER_SKIP;
-		}
-		else {
-			$markers{$def->name} = MARKER_LEAF;
-		}
+		$markers{$def->name} = MARKER_LEAF;
 
 		my $path = $def->get_name_path;
-		my @path_meta = @{$path->meta};
-		for my $ind (0 .. $#path_meta) {
-			if ($path_meta[$ind] eq 'ARRAY') {
+		my $path_meta = $path->meta;
+		for my $ind (0 .. $#$path_meta) {
+			if ($path_meta->[$ind] eq 'ARRAY') {
 				$markers{$path->join($ind - 1)} = MARKER_ARRAY;
 			}
 		}
 	}
 
 	my $error = try sub {
-		$self->_check_recursive($obj, $input, \%markers);
+		$self->_check_recursive($input, \%markers, Form::Tiny::Path->empty);
 	};
 
 	if ($error) {
